#Davis Convex Hull

#Written by Matt Davis matthew.davis@yale.edu
#Please acknowledge or cite where appropriate

#This script will calculate a species contribution to convex hull volume FRic at each time bin.  It is analogous to the nearest neighbor and unique branchlengths code


#This corresponds to proportionalFRic1.0 in the author's own version control system

###################################################################


require(geometry)


#Load in results to get pctraits which will be used for the convex hull and dates
pctraits <- results$pctraits
dim(pctraits)#94 4



#################################################

#Going with an allbin approach
#First run the binning code to get which species are in each bin
#Source code that will timebin allrange
source(file.path(mainDir,"Davis Timebinning.R"), chdir = F)

#Remember that last bin is fake bin
dim(allbin)#3882 35
head(allbin)
tail(allbin)



#############################


#Now we need to loop through each timebin and whether it is that species extinct bin and what its contribution to the convex hull volume is. 

colnames(allbin)

#Add needed columns
allbin$ExtinctBin <- NA
allbin$RelFRic <- 0


#get how many bins
numbins <- length(unique(allbin$Bin))



#Step through each bin
for (i in numbins:1){
	
	
	#Get the taxa in this bin
	thisbintaxa <- as.character(allbin[allbin$Bin==i,"Labels"])
	
	if(i==1){allbin[allbin$Bin==i ,"ExtinctBin"] <- "Surviving"}else{
	
	#And the subsequent bin
	nextbintaxa <- allbin[allbin$Bin==(i-1),"Labels"]
	#And taxa that go extinct before the next bin
	goingtaxa <- thisbintaxa[!thisbintaxa %in% nextbintaxa]
	
	#Then label species that continue on
	allbin[allbin$Bin==i & allbin$Labels %in% nextbintaxa,"ExtinctBin"] <- "Surviving"
	
	#Or  if species goes extinct
	allbin[allbin$Bin==i & allbin$Labels %in% goingtaxa,"ExtinctBin"] <- "Going extinct"
	}#End if else
	
	########################
	#Now on to proportional convex hull volume
	
	#First get the right pc traits for these species
	thesetraits <- pctraits[thisbintaxa,]
	
	#Get the total hull volume for the species in this bin
	#Use FA for volume and Fx for vertices
	conhull <- convhulln(thesetraits, options=c("FA", "Fx"))
	
	#First get the volume 
	binvol <- conhull$vol
	
	#This returns the points of the facets that form the outer surface of the hull so we need to get the unique points which are the row indices in traits
	vertindices <- sort(unique(c(conhull$hull)))
	
	##################
	
	
	
	#Now go through every species in this bin that was a vertex, delete it and recalculate what the convex hull volume would be 
	
	for(j in 1:length(vertindices)){
		
		#Get the species index that is the vertex
		vert <- vertindices[j]
		
		#Drop that species
		droptraits <- thesetraits[-vert,]
		
		#Calculate the hull and get the volume
		drophull <- convhulln(droptraits, options=c("FA"))
	
		#First get the volume 
		dropvol <- drophull$vol

		#Put the relative volume change in
		allbin[allbin$Bin==i & allbin$Labels==thisbintaxa[vert],"RelFRic"] <- (binvol-dropvol)/binvol
		
		
			}#End j taxa loop
		
		
#Print what loop we are on
print(paste("Loop ",i," of ",numbins," just finished",sep=""))		
		
}#End i bin loop

head(allbin)
tail(allbin)



