#Davis Functional Diversity Simple

#Written by Matt Davis matthew.davis@yale.edu
#Please acknowledge or cite where appropriate

#This is a function that will calculate functional diversity index values on different extinction scenarios 


#This corresponds to simplefd1.0 in the author's own version control system



simplefd <- function(traits2=traits2,pctraits=pctraits, therematrix=therematrix,scenario=scenario){

require(FD)	
require(cluster)
require(picante)
	
#Bookkeeping
alltaxa <- rownames(traits2)
numtraits <- dim(pctraits)[2]
numbins <- dim(therematrix)[1]


	
#################################################

#################################################	
	#Make the distance matrix for the tree, don't want standardization because we already standardized and weighted
	
	dist1 <- daisy(traits2, metric="euclidean",stand=F) 
	
	
	#Now make the tree
	tree <- hclust(dist1,method="average")
	#plot(tree)
	
	#Run the tree through the Xtree function which breaks up all the branch lengths.
	#Turn into a phylo object
	ftree <- as.phylo(tree)
	
#################################################
#################################################



	#Make the matrix that will be filled up with each iteration of the functional analysis
	data1 <- as.data.frame(matrix(data=NA, nrow=numbins,ncol=5,dimnames=list(c(1:numbins),c("SpeciesRichness","RelRich", "FD", "FRic","Scenario"))))


#################################################

#Actual calculation of functional diversity indices
	
###############################################		
		
	
	#Tree based FD

	#Use the phylogenetic diversity function in picante.  Don't include root to follow Petchey and Gaston's modified FD measure
	FD1 <- pd(therematrix, ftree, include.root=F)
	
	#Put it in the results matrix
	data1$SpeciesRichness <- FD1$SR
	
	#Standardize FD
	data1$FD <- FD1$PD/max(FD1$PD,na.rm=T)

	
#################################################


#FRic

	#Create a vector for the absolute values to go in
	absvol <- rep(NA,numbins)
	
	#Put this here to switch between the right start point for sim and actual therematrix's. Start at 7 in simulations to avoid a qhull snag
	startpoint <- min(which(rowSums(therematrix)>(numtraits+2)))
	
	
	
	#Loop through each timebin
	for(bb in startpoint:numbins){
		
		#Get the species in this bin
		thisbintaxa <- alltaxa[which(therematrix[bb,]==1)]
		numthisbintaxa <- length(thisbintaxa)
	
		#Get their traits
		thesetraits <- pctraits[thisbintaxa,]
	
		#Get the total hull volume for the species in this bin
		#Use FA for volume and Fx for vertices
		conhull <- convhulln(thesetraits, options=c("FA"))
	
		#First get the volume 
		absvol[bb] <- conhull$vol
	
		}#End bb loop 
	
		#Now make the the convex hull volumes relative
		data1$FRic <- absvol/absvol[numbins]
	
		 
	
#################################################
#################################################
	
	#Fill in relative species richness
	data1$RelRich <- data1$SpeciesRichness/data1$SpeciesRichness[numbins]
	
	#And what iteration/scenario it was
	data1[,"Scenario"] <- scenario
	
	
	
	#head(data1)
	#tail(data1)
	
	return(data1)
	
	}#End function