#Davis Nearest Neighbor

#Written by Matt Davis matthew.davis@yale.edu
#Please acknowledge or cite where appropriate

#This is a loop of code that will take allrange and calculate the top three closest nearest neighbors for each species through time.It will also calculate the distance from each species in a bin to the bin centroid. It will also calculate distances in transformed PCA space, not standard euclidean space. Results will be the same as euclidean distance unless fewer axes are used. m (number of axes to keep) is imported before function call from Function results
#This version will also add in the species pair that the shortest distance actually is.

#This corresponds to nearestneighbor1.4 in the author's own version control system



######################################################
require(cluster)
require(plyr)



#Pull out the traits
colnames(allrange)
traits <- allrange[,c(9,11:19)]
traits

#Set up the weights.  Weighting so mass, diet, and locomotion/sociality all get a third
#groupweight <- 1/3
#weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))
weights <- weights


	
#Bookkeeping
traits <- as.matrix(traits)

	
##########################


#Need to standardize the trait matrix here as I am not doing it later

#Z transform the data so they are standardized 
#Traits transformed so they have a sd of 1 and mean of 0

	df <- traits
	ndf <- df
for(i in 1:dim(df)[1]){
	for(j in 1:dim(df)[2]){
		ndf[i,j] <- (df[i,j]-mean(df[,j], na.rm=T))/sd(df[,j], na.rm=T)
	}#End j loop
	}#End i loop




#Check
#apply(ndf,2,sd)
#apply(ndf,2,mean)

#Now apply weighting to the traits
#This transposes the matrix so that columns are rows and then transposes it back again
ndf2 <- t(t(ndf)*weights)

#Check the standard deviations.  They were 1 before and they should be the weights now
#summary(ndf2)
#apply(ndf2,2,sd)

traits2 <- ndf2

####################	

#Make sure you have a allrange

#After z transformation and weighting, put the traits back into allrange and run binning code to get the average weighted trait values per time bin

#Now do the pca 
#Don't want centering or scaling because I just did that above
pca1 <- prcomp(traits2,center=F,scale=F)

#Pull PCA coordinates out and splice them into allrange. Don't need uncertaRemoving trait uncertainty columns
colnames(allrange)
colnames(traits2)

allrange <- cbind(allrange[,1:8],pca1$x[,1:m],allrange[,30:dim(allrange)[2]])


head(allrange)
summary(allrange)


#Source code that will timebin allrange
source(file.path(mainDir,"Davis Timebinning.R"), chdir = F)

dim(allbin)#3882 18 
head(allbin)
tail(allbin)

#Calculate mean traits (centroid) per bin since I have gone this far

colnames(allbin)

#Now compute the mean trait values for each bin
meanbin <- ddply(allbin, .(Bin,Age), function(x)colMeans(x[,c(9:(8+m))],na.rm=T))

#Rearrange a bit
head(meanbin)
colnames(meanbin)
meanbin <- cbind(meanbin[,3:(2+m)],meanbin[,1:2])
head(meanbin)

#Add rownames
numbins <- dim(meanbin)[1]
rownames(meanbin) <- paste("meantraitbin_",seq(1:numbins),sep="")
meanbin

############################

#Now that allbin is prepped, need to calculate distance matrix off of allrange

#Make the distance matrix, don't want standardization because we already standardized and weighted and PCA'd. Using significant PCA axes as traits
	
traits3 <- 	pca1$x[,1:m]
	
dist1 <- as.matrix(dist(traits3, method="euclidean",diag=T,upper=T))
dim(dist1)#94 94
dist1

#Figure out distance to centroid
meanbin2 <- meanbin[,1:m]
meanbin3 <- rbind(traits3,meanbin2)

totcentroiddist <- as.matrix(dist(meanbin3, method="euclidean",diag=T,upper=T))
#head(totcentroiddist)

#Ok, now I need to go through each species in each bin and calculate the three smallest distances


allbin$Dist1 <- NA
allbin$Dist2 <- NA
allbin$Dist3 <- NA
allbin$Dist1Taxa <- NA
allbin$Dist1Pair <- NA
allbin$PairExtinct <- NA
allbin$DistCent <- NA

head(allbin)


numbins <- length(unique(allbin$Bin))


for(i in 1:numbins){
	#Now for each bin
	thisbin <- allbin[allbin$Bin==i,]
	
	#Pull out only those species in the distance matrix that are in this bin
	thisdist <- dist1[rownames(dist1) %in% thisbin$Labels,colnames(dist1) %in% thisbin$Labels]
	
	#dim(thisdist)#53 53
	#dim(thisbin)#53 checks out
	
	#Also pull out column of centroid distance matrix for this bin
	thiscentroid <- totcentroiddist[,paste("meantraitbin_",i,sep="")]
	
	#Get the number of taxa
	alltaxa <- sort(unique(thisbin$Labels))
	numtaxa <- length(alltaxa)
	
	#Now go through each taxa
	for(j in 1:numtaxa){
		#Pull out the ordered taxa taxa distances
		taxadist <- sort(thisdist[rownames(thisdist)==alltaxa[j],])
		
		#Now put the shortest three into allbin
		#The shortest is actually zero since it is a species to itself so skip that one
		allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"Dist1"] <- taxadist[2]
		allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"Dist2"] <- taxadist[3]
		allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"Dist3"] <- taxadist[4]
		
		#And taxa that shortest distance is to
		allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"Dist1Taxa"] <- names(taxadist[2])
		
		#And combine it together to make the pair
		distpair <- sort(c(names(taxadist[2]),as.character(alltaxa[j])))
		allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"Dist1Pair"] <- paste(distpair[1],distpair[2],sep="-")
		
		#Also figure out if one or both of the species in the pair is extinct
		pairstatus <- allbin[allbin$Labels %in% distpair & allbin$Bin==i,"Status"]
		extinctpair <- pairstatus[pairstatus=="Extinct"]
		#Now fill it up based on how many species in the pair are extinct
		if(length(extinctpair)==0){allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"PairExtinct"] <- "None"}else 
		
		if(length(extinctpair)==1){allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"PairExtinct"] <- "One"}else{
		
		allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"PairExtinct"] <- "Both"	
		}
		
		
		
		#And distance to centroid
		
		allbin[allbin$Labels==alltaxa[j]&allbin$Bin==i,"DistCent"] <- thiscentroid[alltaxa[j]]
		
		
		
		
	}#End j taxa loop


	
	
} #end i bins loop
