#Davis Unique Branchlengths

#Written by Matt Davis matthew.davis@yale.edu
#Please acknowledge or cite where appropriate

#This code looks at the unique contribution to the functional tree that each species has in each bin.

#This corresponds to uniquebranchlengths1.2 in the author's own version control system

######################################################
require(cluster)
library(picante)
#require(plyr)


#First run the binning code to get which species are in each bin
#Source code that will timebin allrange
source(file.path(mainDir,"Davis Timebinning.R"), chdir = F)

#Remember that last bin is fake bin
dim(allbin)#3882 35
head(allbin)
tail(allbin)


#########

#Ok lets get distance down and create the tree

#Pull out the traits
colnames(allrange)
traits <- allrange[,c(9,11:19)]
traits

#Set up the weights.  Weighting so mass, diet, and locomotion/sociality all get a third
#groupweight <- 1/3
#weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))
	
#Bookkeeping
traits <- as.matrix(traits)
	
#################################################

#Z transform the data so they are standardized 
#Traits transformed so they have a sd of 1 and mean of 0

	df <- traits
	ndf <- df
for(i in 1:dim(df)[1]){
	for(j in 1:dim(df)[2]){
		ndf[i,j] <- (df[i,j]-mean(df[,j], na.rm=T))/sd(df[,j], na.rm=T)
	}#End j loop
	}#End i loop

#Check
#apply(ndf,2,sd)
#apply(ndf,2,mean)

#Now apply weighting to the traits. This transposes the matrix so that columns are rows and then transposes it back again
ndf2 <- t(t(ndf)*weights)

#Check the standard deviations.  They were 1 before and they should be the weights now
#summary(ndf2)
#apply(ndf2,2,sd)

traits2 <- ndf2

##########

#Make the distance matrix, don't want standardization because we already standardized and weighted
	
dist1 <- daisy(traits2, metric="euclidean",stand=F) 
	
	
#Now make the tree
tree <- hclust(dist1,method="average")
#plot(tree)

ftree <- as.phylo(tree)

#Get the total length of the tree
totallength <- sum(ftree$edge.length)

#############################


#Now we need to loop through each timebin and calculate the pendant edge branchlength for each species and whether it is that species extinct bin

colnames(allbin)

#Add needed columns
allbin$ExtinctBin <- NA
allbin$RelUniq <- NA
allbin$RelFD <- NA

#get how many bins
numbins <- length(unique(allbin$Bin))

#And the total list of taxa
alltaxa <- sort(unique(allbin$Labels)) 


#Dropping tips automatically drops the root so any FD type calculations that don't include the length of the root edge should be good

#Then step through each one
for (i in numbins:1){
	
	
	#Get the taxa in this bin
	thisbintaxa <- as.character(allbin[allbin$Bin==i,"Labels"])
	
	if(i==1){allbin[allbin$Bin==i ,"ExtinctBin"] <- "Surviving"}else{
	
	#And the subsequent bin
	nextbintaxa <- allbin[allbin$Bin==(i-1),"Labels"]
	#And taxa that go extinct before the next bin
	goingtaxa <- thisbintaxa[!thisbintaxa %in% nextbintaxa]
	
	#Then label species that continue on
	allbin[allbin$Bin==i & allbin$Labels %in% nextbintaxa,"ExtinctBin"] <- "Surviving"
	
	#Or  if species goes extinct
	allbin[allbin$Bin==i & allbin$Labels %in% goingtaxa,"ExtinctBin"] <- "Going extinct"
	}#End if else
	
	########################
	#Now on to uniqueness and branchlengths
	
	#First get the branchlength for the tree in this bin
	#What species aren't in this bin's tree?
	droptaxa <- as.character(alltaxa[!alltaxa %in% thisbintaxa])
	if(length(droptaxa)==0){bintree <- ftree}else{bintree <- drop.tip(ftree, droptaxa)}
	
	binlength <- sum(bintree$edge.length)
	
	#####
	
	#Also get the functional equivalent of evolutionary distiveness
	FDp <- evol.distinct(bintree,type="fair.proportion",scale=F)
	colnames(FDp) <- c("Labels","FDp")
	#Add a relative branch length, don't multiply by 100 to keep consistant with Uniqueness
	FDp$FDprel <- (FDp$FDp/sum(FDp$FDp))
	#sum(FDp$FDprel)#Should equal 1
	
	
	
	#####
	#Now go through every species and see what their pendant edge would be and put in their Proportional FD too
	for(j in 1:length(thisbintaxa)){
		
		#Drop that species
		minustree <- drop.tip(bintree, thisbintaxa[j])
		
		#Get the length of that tree
		neglength <- sum(minustree$edge.length)
	
		#How much of the bin tree is that
		allbin[allbin$Bin==i & allbin$Labels==thisbintaxa[j],"RelUniq"] <- (binlength-neglength)/binlength
	
		#And what is the proportional FD
		allbin[allbin$Bin==i & allbin$Labels==thisbintaxa[j],"RelFD"] <- FDp[FDp$Labels==thisbintaxa[j],"FDprel"]	
	
	}#End j taxa loop
		
}#End i bin loop

####

head(allbin)
tail(allbin)

#Diagnostic
#abin <- allbin[allbin$Bin==13,]
#sum(abin$RelFD)#1
#sum(abin$RelUniq)#.76

#hist(abin$RelFD-abin$RelUniq)